@@ -1,5 +1,5 @@
-#!/2022Q2/bin/perl -w
-# Copyright © 2021-2022 Jamie Zawinski <jwz@jwz.org>
+#!/usr/bin/perl -w
+# Copyright © 2021-2023 Jamie Zawinski <jwz@jwz.org>
 #
 # Permission to use, copy, modify, distribute, and sell this software and its
 # documentation for any purpose is hereby granted without fee, provided that
@@ -29,7 +29,7 @@ BEGIN { eval 'use LWP::Simple' }
 
 my $progname = $0; $progname =~ s@.*/@@g;
 $progname =~ s@\.pl$@@g;
-my ($version) = ('$Revision: 1.5 $' =~ m/\s(\d[.\d]+)\s/s);
+my ($version) = ('$Revision: 1.9 $' =~ m/\s(\d[.\d]+)\s/s);
 
 my $verbose = 0;
 my $url_template = undef;
@@ -42,17 +42,25 @@ my %cached_files;
 # script exits, even if abnormally.
 #
 my %rm_f;
-END {
-  my $exit = $?;
-  my @rm_f = keys %rm_f;
-  unlink @rm_f if (@rm_f);
-  $? = $exit;  # Don't clobber this script's exit code.
+END { rmf(); }
+
+sub rmf() {
+  foreach my $f (sort keys %rm_f) {
+    if (-e $f) {
+      print STDERR blurb() . "rm $f\n" if ($verbose > 1);
+      unlink $f;
+    }
+  }
+  %rm_f = ();
 }
 
 sub signal_cleanup($) {
   my ($s) = @_;
-  print STDERR "$progname: SIG$s\n" if ($verbose > 1);
-  exit (1);  # This causes END{} to run.
+  print STDERR blurb() . "SIG$s\n" if ($verbose > 1);
+  rmf();
+  # Propagate the signal and die. This does not cause END to run.
+  $SIG{$s} = 'DEFAULT';
+  kill ($s, $$);
 }
 
 $SIG{TERM} = \&signal_cleanup;  # kill
@@ -63,6 +71,12 @@ $SIG{ABRT} = \&signal_cleanup;
 $SIG{PIPE} = \&signal_cleanup;
 $SIG{HUP}  = \&signal_cleanup;
 
+# Add the file to the rm_f list, since you can't export hash variables.
+sub rm_atexit($) {
+  my ($file) = @_;
+  $rm_f{$file} = 1;
+}
+
 
 sub blurb() {
   return ("$progname: " . strftime('%H:%M:%S', localtime) .
@@ -84,10 +98,22 @@ sub sanity_check_lwp() {
   $sanity_checked_p = 1;
   my $url1 = 'https://www.mozilla.org/';
   my $url2 =  'http://www.mozilla.org/';
-  my $body = (LWP::Simple::get($url1) || '');
+  my $body = LWP::Simple::get($url1) || '';
+
   if (length($body) < 10240) {
+    sleep (3);
+    $body = LWP::Simple::get($url1) || '';
+  }
+
+  if (length($body) < 10240) {
+    $body = LWP::Simple::get($url2) || '';
+
+    if (length($body) < 10240) {
+      sleep (3);
+      $body = LWP::Simple::get($url2) || '';
+    }
+
     my $err = "";
-    $body = (LWP::Simple::get($url2) || '');
     if (length($body) < 10240) {
       $err = "Perl is broken: neither HTTP nor HTTPS URLs work.";
     } else {
@@ -178,11 +204,16 @@ sub apply_template($$$) {
     if    ($s =~ m/^\$?x$/si) { $s = $x; }       # {$x}
     elsif ($s =~ m/^\$?y$/si) { $s = $y; }       # {$y}
     elsif ($s =~ m/^\$?z$/si) { $s = $z; }       # {$z}
+    elsif ($s =~ m/^\$?r$/si) { $s = ''; }       # {$r} -- "" or "@2x"
     elsif ($s =~ m/^([a-z\d])-([a-z\d])$/si) {   # {a-c}
       my ($from, $to) = (ord($1), ord($2));
       my @a = ( $from .. $to );
-      $s = chr($a[rand() * @a]);
-    } else { error ("unparsable substitution: {$s} in $url"); }
+      $s = chr($a[rand() * @a]); }
+    elsif ($s =~ m/^\$?s$/si) {			 # {$s} means {a-d} only
+      my ($from, $to) = (ord('a'), ord('d'));
+      my @a = ( $from .. $to );
+      $s = chr($a[rand() * @a]); }
+    else { error ("unparsable substitution: {$s} in $url"); }
     $s;
   }%gsexi;
 
@@ -200,6 +231,7 @@ sub load_tile($$$$) {
 
   # Sanitize the template into a directory name
   my $site = lc($url_template);
+  $site =~ s/[?#].*$//s;
   my ($suf) = ($site =~ m@\.([^./?#]+)$@si);
   error ("no suffix in url template: $url_template") unless $suf;
 
@@ -227,7 +259,7 @@ sub load_tile($$$$) {
   # Hit the network.
   #
   my $tmpfile = sprintf("%s/%08x.%s", $cache_dir, rand() * 0xFFFFFFFF, $suf);
-  $rm_f{$tmpfile} = 1;
+  rm_atexit ($tmpfile);
 
   acquire_lock();
   print STDERR blurb() . "downloading $url\n" if ($verbose > 1);
